@@ -1,6 +1,6 @@
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* kadmin/ktutil/ktutil.h */
 /*
- * kadmin/ktutil/ktutil.h
- *
  * Copyright 1995 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
@@ -8,7 +8,7 @@
  *   require a specific license from the United States Government.
  *   It is the responsibility of any person or organization contemplating
  *   export to obtain such a license before exporting.
- * 
+ *
  * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
  * distribute this software and its documentation for any purpose and
  * without fee is hereby granted, provided that the above copyright
@@ -22,7 +22,6 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
- * 
  */
 
 typedef struct _krb5_kt_list {
@@ -35,28 +34,23 @@ krb5_error_code ktutil_free_kt_list (krb5_context, krb5_kt_list);
 krb5_error_code ktutil_delete (krb5_context, krb5_kt_list *, int);
 
 krb5_error_code ktutil_add (krb5_context,
-			    krb5_kt_list *,
-			    char *,
-			    krb5_kvno,
-			    char *,
-			    int);
+                            krb5_kt_list *,
+                            char *,
+                            krb5_kvno,
+                            char *,
+                            int);
 
 krb5_error_code ktutil_read_keytab (krb5_context,
-				    char *,
-				    krb5_kt_list *);
+                                    char *,
+                                    krb5_kt_list *);
 
 krb5_error_code ktutil_write_keytab (krb5_context,
-				     krb5_kt_list,
-				     char *);
+                                     krb5_kt_list,
+                                     char *);
 
-#ifdef KRB5_KRB4_COMPAT
 krb5_error_code ktutil_read_srvtab (krb5_context,
-				    char *,
-				    krb5_kt_list *);
-krb5_error_code ktutil_write_srvtab (krb5_context,
-				     krb5_kt_list,
-				     char *);
-#endif
+                                    char *,
+                                    krb5_kt_list *);
 
 void ktutil_add_entry (int, char *[]);
 
