@@ -51,7 +51,7 @@
  */
 #define KRB5_MAJOR_RELEASE 1
 #define KRB5_MINOR_RELEASE 12
-#define KRB5_PATCHLEVEL 0
+#define KRB5_PATCHLEVEL 1
 /* #undef KRB5_RELTAIL */
-#define KRB5_RELDATE "20131210"
-#define KRB5_RELTAG "krb5-1.12-final"
+#define KRB5_RELDATE "20140115"
+#define KRB5_RELTAG "krb5-1.12.1-final"
