@@ -1,7 +1,6 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* util/support/plugins.c - Plugin module support functions */
 /*
- * util/support/plugins.c
- *
  * Copyright 2006, 2008 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
@@ -23,9 +22,6 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
- *
- *
- * Plugin module support, and shims around dlopen/whatever.
  */
 
 #include "k5-plugin.h"
@@ -159,8 +155,8 @@ int closedir(DIR *dp)
 {
     if (!dp) return 0;
     _findclose(dp->handle);
-    if (dp->dir) free(dp->dir);
-    if (dp) free(dp);
+    free(dp->dir);
+    free(dp);
 
     return 0;
 }
@@ -176,8 +172,10 @@ krb5int_open_plugin (const char *filepath, struct plugin_file_handle **h, struct
 
     if (!err) {
         if (stat (filepath, &statbuf) < 0) {
-            Tprintf ("stat(%s): %s\n", filepath, strerror (errno));
             err = errno;
+            Tprintf ("stat(%s): %s\n", filepath, strerror (err));
+            k5_set_error(ep, err, _("unable to find plugin [%s]: %s"),
+                         filepath, strerror(err));
         }
     }
 
@@ -269,10 +267,11 @@ krb5int_open_plugin (const char *filepath, struct plugin_file_handle **h, struct
             if (handle == NULL) {
                 const char *e = dlerror();
                 if (e == NULL)
-                    e = "unknown failure";
+                    e = _("unknown failure");
                 Tprintf ("dlopen(%s): %s\n", filepath, e);
                 err = ENOENT; /* XXX */
-                krb5int_set_error (ep, err, "%s", e);
+                k5_set_error(ep, err, _("unable to load plugin [%s]: %s"),
+                             filepath, e);
             }
         }
 
@@ -294,7 +293,7 @@ krb5int_open_plugin (const char *filepath, struct plugin_file_handle **h, struct
         if (handle == NULL) {
             Tprintf ("Unable to load dll: %s\n", filepath);
             err = ENOENT; /* XXX */
-            krb5int_set_error (ep, err, "%s", "unable to load dll");
+            k5_set_error(ep, err, _("unable to load DLL [%s]"), filepath);
         }
 
         if (!err) {
@@ -310,6 +309,7 @@ krb5int_open_plugin (const char *filepath, struct plugin_file_handle **h, struct
 
     if (!err && !got_plugin) {
         err = ENOENT;  /* no plugin or no way to load plugins */
+        k5_set_error(ep, err, _("plugin unavailable: %s"), strerror(err));
     }
 
     if (!err) {
@@ -317,7 +317,7 @@ krb5int_open_plugin (const char *filepath, struct plugin_file_handle **h, struct
         htmp = NULL;  /* h takes ownership */
     }
 
-    if (htmp != NULL) { free (htmp); }
+    free(htmp);
 
     return err;
 }
@@ -341,7 +341,7 @@ krb5int_get_plugin_sym (struct plugin_file_handle *h,
                 e = "unknown failure";
             Tprintf ("dlsym(%s): %s\n", csymname, e);
             err = ENOENT; /* XXX */
-            krb5int_set_error(ep, err, "%s", e);
+            k5_set_error(ep, err, "%s", e);
         }
     }
 #endif
@@ -354,9 +354,9 @@ krb5int_get_plugin_sym (struct plugin_file_handle *h,
         sym = GetProcAddress(h->hinstPlugin, csymname);
         if (sym == NULL) {
             const char *e = "unable to get dll symbol"; /* XXX copy and save away */
-            Tprintf ("GetProcAddress(%s): %s\n", csymname, GetLastError());
+            Tprintf ("GetProcAddress(%s): %i\n", csymname, GetLastError());
             err = ENOENT; /* XXX */
-            krb5int_set_error(ep, err, "%s", e);
+            k5_set_error(ep, err, "%s", e);
 
             dw = GetLastError();
             if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
@@ -551,7 +551,7 @@ krb5int_get_plugin_filenames (const char * const *filebases, char ***filenames)
         tempnames = NULL;
     }
 
-    if (tempnames) { krb5int_free_plugin_filenames (tempnames); }
+    krb5int_free_plugin_filenames(tempnames);
 
     return err;
 }
@@ -602,7 +602,7 @@ krb5int_open_plugin_dirs (const char * const *dirnames,
                     if (!err) { handle = NULL; }  /* h takes ownership */
                 }
 
-                if (filepath != NULL) { free (filepath); }
+                free(filepath);
                 if (handle   != NULL) { krb5int_close_plugin (handle); }
             }
         } else {
@@ -637,7 +637,7 @@ krb5int_open_plugin_dirs (const char * const *dirnames,
                     }
                 }
 
-                if (filepath  != NULL) { free (filepath); }
+                free(filepath);
                 if (handle    != NULL) { krb5int_close_plugin (handle); }
             }
 
@@ -727,7 +727,7 @@ krb5int_get_plugin_dir_data (struct plugin_dir_handle *dirhandle,
         p = NULL; /* ptrs takes ownership */
     }
 
-    if (p != NULL) { free (p); }
+    free(p);
 
     return err;
 }
@@ -786,7 +786,7 @@ krb5int_get_plugin_dir_func (struct plugin_dir_handle *dirhandle,
         p = NULL; /* ptrs takes ownership */
     }
 
-    if (p != NULL) { free (p); }
+    free(p);
 
     return err;
 }
